@@ -1,14 +1,10 @@
-;;; ebib-utils.el --- Part of Ebib, a BibTeX database manager  -*- lexical-binding: t -*-
+;;; ebib-notes.el --- Part of Ebib, a BibTeX database manager  -*- lexical-binding: t -*-
 
 ;; Copyright (c) 2003-2016 Joost Kremers
 ;; All rights reserved.
 
 ;; Author: Joost Kremers <joostkremers@fastmail.fm>
 ;; Maintainer: Joost Kremers <joostkremers@fastmail.fm>
-;; Created: 2016
-;; Version: 2.6
-;; Keywords: text bibtex
-;; Package-Requires: ((dash "2.5.0") (emacs "24.3"))
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
@@ -56,7 +52,7 @@ one file per note, using the options `ebib-notes-directory',
   :type '(choice (const :tag "Use multiple notes files" nil)
                  (file :tag "Notes File")))
 
-(defcustom ebib-notes-symbol "[N]"
+(defcustom ebib-notes-symbol "N"
   "Symbol used to indicate the presence of a note for the current entry.
 If there is a note for the current entry, this symbol is
 displayed in the mode line of the entry buffer after the entry
@@ -93,61 +89,41 @@ used instead."
 
 (defcustom ebib-notes-template "* %T\n:PROPERTIES:\n%K\n:END:\n>|<\n"
   "Template for a note entry in the notes file.
-New notes are created on the basis of this template, which can
-contain the following directives:
+New notes are created on the basis of this template.  The
+template can contain format specifiers consisting of a percent
+sign and a character.  These specifiers are defined by
+`ebib-notes-template-specifiers'.  Note that the `%K' specifier
+must be present in the template and should be replaced by an
+identifier that is unique for the entry.  This identifier is used
+to retrieve the note.  Without it, Ebib is not able to determine
+whether an entry has a note or not.
 
-%T  : the title of the entry
-%K  : the unique identifier of the note
->|< : the position of the cursor
-
-If all notes are stored in a single notes file, the %K directive
-must be present in the template, since Ebib uses it to identify
-the note.  Without it, Ebib cannot determine whether an entry
-already has a note or not.  The identifier is created on the
-basis of the entry key using the function in the option
-`ebib-notes-identifier-function'.  The %T directive is replaced
-with the title of the note, which is created using the function
-in `ebib-notes-title-function'.  The sequence '>|<' is removed
-from the template and the cursor will be positioned in its
-place."
+The template can also contain the string \">|<\" to indicate the
+position where the cursor is to be placed when creating a new
+note."
   :group 'ebib-notes
   :type '(string :tag "Note template"))
 
-(defcustom ebib-notes-title-function 'ebib-notes-create-org-title
-  "Function to create the title for a notes entry.
-This function is used to fill the %T directive in
-`ebib-notes-template'.  It should take one argument, the key of
-the entry for which a title is to be created."
+(defcustom ebib-notes-template-specifiers '((?K . ebib-create-org-identifier)
+                                        (?T . ebib-create-org-title)
+                                        (?L . ebib-create-org-link)
+                                        (?F . ebib-create-org-file-link)
+                                        (?D . ebib-create-org-doi-link)
+                                        (?U . ebib-create-org-url-link))
+  "Specifiers used in `ebib-notes-template'.
+Each specifier consists of a character (which is preceded by a
+percent sign in `ebib-notes-template') and a symbol, which
+either names a function to be executed or a variable, which
+should hold a string.  If a function, it should take two
+arguments, the entry key and the database, and should return a
+string that is substituted for the specifier in the template.
+
+Note that the `K' specifier should not be removed, since it is
+used to create an identifier for the note."
   :group 'ebib-notes
-  :type 'function)
-
-(defun ebib-notes-create-org-title (key)
-  "Return a title for an orgmode note for KEY.
-The title is formed from the author(s) or editor(s) of the entry,
-its year and its title.  Newlines are removed from the resulting
-string."
-  (let ((author (or (ebib-db-get-field-value "author" key ebib--cur-db 'noerror 'unbraced 'xref)
-                    (ebib-db-get-field-value "editor" key ebib--cur-db 'noerror 'unbraced 'xref)
-                    "(No Author)"))
-        (year (or (ebib-db-get-field-value "year" key ebib--cur-db 'noerror 'unbraced 'xref)
-                  "????"))
-        (title (or (ebib-db-get-field-value "title" key ebib--cur-db 'noerror 'unbraced 'xref)
-                   "(No Title)")))
-    (replace-regexp-in-string "\n" "" (format "%s (%s): %s" author year title))))
-
-(defcustom ebib-notes-identifier-function 'ebib-notes-create-org-identifier
-  "Function to create the identifier of a note.
-This function should take the key of the entry as argument and
-should return as string that uniquely identifies the entry in the
-notes file."
-  :group 'ebib-notes
-  :type 'function)
-
-(defun ebib-notes-create-org-identifier (key)
-  "Create a unique identifier for KEY for use in an org file.
-The key is prepended with the string \"Custom_id:\", so that it
-can be used in a :PROPERTIES: block."
-  (format ":Custom_id: %s" key))
+  :type '(repeat (cons :tag "Specifier"
+                       (character :tag "Character")
+                       (symbol :tag "Function or variable"))))
 
 (defcustom ebib-notes-search-note-before-hook '(widen)
   "Hook run before searching for a note.
@@ -159,11 +135,11 @@ This hook is also run when a new note is being created."
   :group 'ebib
   :type 'hook)
 
-(defcustom ebib-notes-open-note-after-hook '(org-back-to-heading org-narrow-to-subtree)
+(defcustom ebib-notes-open-note-after-hook '(org-back-to-heading org-narrow-to-subtree org-show-subtree)
   "Hook run after a note is found.
 This hook is only used when notes are stored in a common notes
-file.  It can be used to position the cursor after the note has
-been found.
+file.  It can be used to prepare the note for display, position
+the cursor, etc.
 
 This hook is not run when a new note is created, see
 `ebib-notes-new-note-hook'."
@@ -175,13 +151,11 @@ This hook is not run when a new note is created, see
   :group 'ebib-notes
   :type 'hook)
 
-(defun ebib--notes-create-new-note-from-template (key)
-  "Create a new note for KEY.
+(defun ebib--notes-fill-template (key db)
+  "Create a new note for KEY in DB.
 Return a cons of the new note as a string and a position in this
 string where point should be located."
-  (let* ((note (format-spec ebib-notes-template
-                            `((?K . ,(funcall ebib-notes-identifier-function key))
-                              (?T . ,(funcall ebib-notes-title-function key)))))
+  (let* ((note (ebib-format-template ebib-notes-template ebib-notes-template-specifiers key db))
          (point (string-match-p ">|<" note)))
     (if point
         (setq note (replace-regexp-in-string ">|<" "" note))
@@ -191,8 +165,9 @@ string where point should be located."
 (defun ebib--notes-exists-note (key)
   "Return t if a note exists for KEY."
   (if ebib-notes-use-single-file
-      (with-current-buffer (ebib--notes-buffer)
-        (ebib--notes-locate-note key))
+      (if (file-writable-p ebib-notes-use-single-file)
+          (with-current-buffer (ebib--notes-buffer)
+            (ebib--notes-locate-note key)))
     (file-readable-p (ebib--create-notes-file-name key))))
 
 ;;; one file per note
@@ -210,50 +185,51 @@ name is fully qualified by prepending the directory in
                    key)
           ebib-notes-file-extension))
 
-(defun ebib--notes-open-notes-file-for-entry (key)
-  "Open or create a notes file for KEY."
-  (let* ((filename (expand-file-name (ebib--create-notes-file-name (ebib--cur-entry-key))))
+(defun ebib--notes-open-notes-file-for-entry (key db)
+  "Open or create a notes file for KEY in DB."
+  (let* ((filename (expand-file-name (ebib--create-notes-file-name key)))
          (new (not (file-exists-p filename))))
     (if (not (file-writable-p filename))
         (error "[Ebib] Could not create file `%s' " filename))
     (ebib-lower)
     (find-file filename)
     (if new
-        (let ((initial-contents (ebib--notes-create-new-note-from-template key)))
-          (insert (car initial-contents))
-          (forward-char (cdr initial-contents))))))
+        (let ((note (ebib--notes-fill-template key db)))
+          (insert (car note))
+          (forward-char (cdr note))))))
 
 ;;; common notes file
 
 (defun ebib--notes-buffer ()
   "Return the buffer containing the notes file.
 If the file has not been opened yet, open it, creating it if
-necessary.  Note that this function assumes that the
-`ebib-notes-use-single-file' is set and will raise an error if not.  An
-error is also raised if the location for the notes file is not
-accessible to the user."
+necessary.  Note that this function assumes that
+`ebib-notes-use-single-file' is set.  An error is raised if it is
+not.  An error is also raised if the location for the notes file
+is not accessible to the user."
   (unless ebib-notes-use-single-file
     (error "[Ebib] No notes file defined"))
-  (if (not (file-writable-p ebib-notes-use-single-file))
-      (error "[Ebib] Cannot read or create notes file"))
+  (unless (file-writable-p ebib-notes-use-single-file)
+    (error "[Ebib] Cannot read or create notes file"))
   (find-file-noselect ebib-notes-use-single-file))
 
 (defun ebib--notes-locate-note (key)
   "Locate the note identified by KEY.
-Convert KEY into an identifier using the function in
-`ebib-notes-identifier-function' and search this identifier.  If
-found, return its location as a buffer position, otherwise return
-nil.  The search is performed in the current buffer, so the notes
-buffer must be made active before calling this function.
+Convert KEY into an identifier using the function associated with
+`%K' in `ebib-notes-template-specifiers' and search this
+identifier.  If found, return its location as a buffer position,
+otherwise return nil.  The search is performed in the current
+buffer, so the notes buffer must be made active before calling
+this function.
 
 This function also runs `ebib-notes-search-note-before-hook'."
   (run-hooks 'ebib-notes-search-note-before-hook)
   (save-excursion
     (goto-char (point-min))
-    (search-forward (funcall ebib-notes-identifier-function key) nil t)))
+    (search-forward (funcall (cdr (assoc ?K ebib-notes-template-specifiers)) key nil) nil t)))
 
-(defun ebib--notes-open-common-notes-file (key)
-  "Open the notes file to entry KEY or create a new note."
+(defun ebib--notes-open-common-notes-file (key db)
+  "Open the notes file for entry KEY in DB or create a new note."
   (let ((buf (ebib--notes-buffer)))
     (with-current-buffer buf
       (let ((location (ebib--notes-locate-note key)))
@@ -262,7 +238,7 @@ This function also runs `ebib-notes-search-note-before-hook'."
               (goto-char location)
               (run-hooks 'ebib-notes-open-note-after-hook))
           (goto-char (point-max))
-          (let ((new-note (ebib--notes-create-new-note-from-template key))
+          (let ((new-note (ebib--notes-fill-template key db))
                 (beg (point)))
             (insert (car new-note))
             (goto-char (+ beg (cdr new-note)))
